@@ -23,13 +23,13 @@ import (
 	digest "github.com/opencontainers/go-digest"
 
 	"k8s.io/cli-runtime/pkg/genericclioptions"
-	kcmdutil "k8s.io/kubernetes/pkg/kubectl/cmd/util"
-	"k8s.io/kubernetes/pkg/kubectl/util/templates"
+	kcmdutil "k8s.io/kubectl/pkg/cmd/util"
+	"k8s.io/kubectl/pkg/util/templates"
 
 	"github.com/openshift/api/image/docker10"
 	"github.com/openshift/library-go/pkg/image/dockerv1client"
-	imagereference "github.com/openshift/library-go/pkg/image/reference"
 	"github.com/openshift/library-go/pkg/image/registryclient"
+	"github.com/openshift/oc/pkg/cli/image/imagesource"
 	imagemanifest "github.com/openshift/oc/pkg/cli/image/manifest"
 	"github.com/openshift/oc/pkg/cli/image/workqueue"
 	"github.com/openshift/oc/pkg/helpers/image/dockerlayer"
@@ -93,6 +93,9 @@ type AppendImageOptions struct {
 	DryRun bool
 	Force  bool
 
+	FromFileDir string
+	FileDir     string
+
 	genericclioptions.IOStreams
 }
 
@@ -136,6 +139,9 @@ func NewCmdAppendImage(name string, streams genericclioptions.IOStreams) *cobra.
 
 	flag.BoolVar(&o.Force, "force", o.Force, "If set, the command will attempt to upload all layers instead of skipping those that are already uploaded.")
 
+	flag.StringVar(&o.FileDir, "dir", o.FileDir, "The directory on disk that file:// images will be copied under.")
+	flag.StringVar(&o.FromFileDir, "from-dir", o.FromFileDir, "The directory on disk that file:// images will be read from. Overrides --dir")
+
 	return cmd
 }
 
@@ -184,22 +190,22 @@ func (o *AppendImageOptions) Run() error {
 		}
 	}
 
-	var from *imagereference.DockerImageReference
+	var from *imagesource.TypedImageReference
 	if len(o.From) > 0 {
-		src, err := imagereference.Parse(o.From)
+		src, err := imagesource.ParseReference(o.From)
 		if err != nil {
 			return err
 		}
-		if len(src.Tag) == 0 && len(src.ID) == 0 {
+		if len(src.Ref.Tag) == 0 && len(src.Ref.ID) == 0 {
 			return fmt.Errorf("--from must point to an image ID or image tag")
 		}
 		from = &src
 	}
-	to, err := imagereference.Parse(o.To)
+	to, err := imagesource.ParseReference(o.To)
 	if err != nil {
 		return err
 	}
-	if len(to.ID) > 0 {
+	if len(to.Ref.ID) > 0 {
 		return fmt.Errorf("--to may not point to an image by ID")
 	}
 
@@ -208,9 +214,23 @@ func (o *AppendImageOptions) Run() error {
 	if err != nil {
 		return err
 	}
-	toContext := fromContext.Copy().WithActions("pull", "push")
+	fromOptions := &imagesource.Options{
+		FileDir:         o.FileDir,
+		Insecure:        o.SecurityOptions.Insecure,
+		RegistryContext: fromContext,
+	}
+	if len(o.FromFileDir) > 0 {
+		fromOptions.FileDir = o.FromFileDir
+	}
 
-	toRepo, err := toContext.Repository(ctx, to.DockerClientDefaults().RegistryURL(), to.RepositoryName(), o.SecurityOptions.Insecure)
+	toContext := fromContext.Copy().WithActions("pull", "push")
+	toOptions := &imagesource.Options{
+		FileDir:         o.FileDir,
+		Insecure:        o.SecurityOptions.Insecure,
+		RegistryContext: toContext,
+	}
+
+	toRepo, err := toOptions.Repository(ctx, to)
 	if err != nil {
 		return err
 	}
@@ -227,13 +247,13 @@ func (o *AppendImageOptions) Run() error {
 		fromRepo          distribution.Repository
 	)
 	if from != nil {
-		repo, err := fromContext.Repository(ctx, from.DockerClientDefaults().RegistryURL(), from.RepositoryName(), o.SecurityOptions.Insecure)
+		repo, err := fromOptions.Repository(ctx, *from)
 		if err != nil {
 			return err
 		}
 		fromRepo = repo
 
-		srcManifest, manifestLocation, err := imagemanifest.FirstManifest(ctx, *from, repo, o.FilterOptions.Include)
+		srcManifest, manifestLocation, err := imagemanifest.FirstManifest(ctx, from.Ref, repo, o.FilterOptions.Include)
 		if err != nil {
 			return fmt.Errorf("unable to read image %s: %v", from, err)
 		}
@@ -382,7 +402,7 @@ func (o *AppendImageOptions) Run() error {
 
 				// copy the blob, calculating layer digest if needed
 				var mountFrom reference.Named
-				if from != nil && from.Registry == to.Registry {
+				if from != nil && from.EqualRegistry(to) {
 					mountFrom = fromRepo.Named()
 				}
 				desc, layerDigest, err := copyBlob(ctx, fromBlobs, toBlobs, *layer, o.Out, needLayerDigest, mountFrom)
@@ -414,7 +434,7 @@ func (o *AppendImageOptions) Run() error {
 		return fmt.Errorf("unable to upload the new image manifest: %v", err)
 	}
 	klog.V(4).Infof("Created config JSON:\n%s", configJSON)
-	toDigest, err := imagemanifest.PutManifestInCompatibleSchema(ctx, manifest, to.Tag, toManifests, toRepo.Named(), fromRepo.Blobs(ctx), configJSON)
+	toDigest, err := imagemanifest.PutManifestInCompatibleSchema(ctx, manifest, to.Ref.Tag, toManifests, toRepo.Named(), fromRepo.Blobs(ctx), configJSON)
 	if err != nil {
 		return fmt.Errorf("unable to convert the image to a compatible schema version: %v", err)
 	}
